@@ -1,47 +1,59 @@
-/*
- * -----------------------------------------------------------------
- * $Revision: 1.15 $
- * $Date: 2010/12/01 22:35:26 $
- * ----------------------------------------------------------------- 
+/* -----------------------------------------------------------------
  * Programmer(s): Alan Hindmarsh, Radu Serban and
  *                Aaron Collier @ LLNL
  * -----------------------------------------------------------------
- * Copyright (c) 2002, The Regents of the University of California  
- * Produced at the Lawrence Livermore National Laboratory
- * All rights reserved
- * For details, see the LICENSE file
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
+ * and Southern Methodist University.
+ * All rights reserved.
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
  * -----------------------------------------------------------------
- * This is the implementation file for the optional inputs and     
- * outputs for the IDA solver.                                    
- * -----------------------------------------------------------------
- */
+ * This is the implementation file for the optional inputs and
+ * outputs for the IDA solver.
+ * -----------------------------------------------------------------*/
 
 #include <stdio.h>
 #include <stdlib.h>
 
 #include "ida_impl.h"
-
-#include <sundials/sundials_math.h>
+#include "ida_ls_impl.h"
+#include "sundials/sundials_types.h"
+#include "sundials/sundials_math.h"
 
 #define ZERO    RCONST(0.0)
 #define HALF    RCONST(0.5)
 #define ONE     RCONST(1.0)
 #define TWOPT5  RCONST(2.5)
 
-/* 
+/*
  * =================================================================
  * IDA optional input functions
  * =================================================================
  */
 
-/* 
- * Readability constants
- */
+int IDASetDeltaCjLSetup(void *ida_mem, realtype dcj)
+{
+  IDAMem IDA_mem;
 
-#define lrw  (IDA_mem->ida_lrw)
-#define liw  (IDA_mem->ida_liw)
-#define lrw1 (IDA_mem->ida_lrw1)
-#define liw1 (IDA_mem->ida_liw1)
+  if (ida_mem == NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDASetDeltaCjLSetup",
+                    MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+
+  IDA_mem = (IDAMem) ida_mem;
+
+  if (dcj < ZERO || dcj >= ONE)
+    IDA_mem->ida_dcj = DCJ_DEFAULT;
+  else
+    IDA_mem->ida_dcj = dcj;
+
+  return(IDA_SUCCESS);
+}
 
 int IDASetErrHandlerFn(void *ida_mem, IDAErrHandlerFn ehfun, void *eh_data)
 {
@@ -97,6 +109,144 @@ int IDASetUserData(void *ida_mem, void *user_data)
 
 /*-----------------------------------------------------------------*/
 
+int IDASetEtaFixedStepBounds(void *ida_mem, realtype eta_min_fx,
+                             realtype eta_max_fx)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem == NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDASetEtaFixedStepBounds",
+                    MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+  IDA_mem = (IDAMem) ida_mem;
+
+  /* set allowed value or use default */
+  if (eta_min_fx >= ZERO && eta_min_fx <= ONE)
+    IDA_mem->ida_eta_min_fx = eta_min_fx;
+  else
+    IDA_mem->ida_eta_min_fx = ETA_MIN_FX_DEFAULT;
+
+  if (eta_max_fx >= ONE)
+    IDA_mem->ida_eta_max_fx = eta_max_fx;
+  else
+    IDA_mem->ida_eta_max_fx = ETA_MAX_FX_DEFAULT;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
+int IDASetEtaMax(void *ida_mem, realtype eta_max)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem == NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDASetEtaMax",
+                    MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+  IDA_mem = (IDAMem) ida_mem;
+
+  /* set allowed value or use default */
+  if (eta_max <= ONE)
+    IDA_mem->ida_eta_max = ETA_MAX_DEFAULT;
+  else
+    IDA_mem->ida_eta_max = eta_max;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
+int IDASetEtaMin(void *ida_mem, realtype eta_min)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem == NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDASetEtaMin",
+                    MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+  IDA_mem = (IDAMem) ida_mem;
+
+  /* set allowed value or use default */
+  if (eta_min <= ZERO || eta_min >= ONE)
+    IDA_mem->ida_eta_min = ETA_MIN_DEFAULT;
+  else
+    IDA_mem->ida_eta_min = eta_min;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
+int IDASetEtaLow(void *ida_mem, realtype eta_low)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem == NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDASetEtaLow",
+                    MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+  IDA_mem = (IDAMem) ida_mem;
+
+  /* set allowed value or use default */
+  if (eta_low <= ZERO || eta_low >= ONE)
+    IDA_mem->ida_eta_low = ETA_LOW_DEFAULT;
+  else
+    IDA_mem->ida_eta_low = eta_low;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
+int IDASetEtaMinErrFail(void *ida_mem, realtype eta_min_ef)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem == NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDASetEtaMinErrFail",
+                    MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+  IDA_mem = (IDAMem) ida_mem;
+
+  /* set allowed value or use default */
+  if (eta_min_ef <= ZERO || eta_min_ef >= ONE)
+    IDA_mem->ida_eta_min_ef = ETA_MIN_EF_DEFAULT;
+  else
+    IDA_mem->ida_eta_min_ef = eta_min_ef;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
+int IDASetEtaConvFail(void *ida_mem, realtype eta_cf)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem == NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDASetEtaConvFail",
+                    MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+  IDA_mem = (IDAMem) ida_mem;
+
+  /* set allowed value or use default */
+  if (eta_cf <= ZERO || eta_cf >= ONE)
+    IDA_mem->ida_eta_cf = ETA_CF_DEFAULT;
+  else
+    IDA_mem->ida_eta_cf = eta_cf;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
 int IDASetMaxOrd(void *ida_mem, int maxord)
 {
   IDAMem IDA_mem;
@@ -121,9 +271,9 @@ int IDASetMaxOrd(void *ida_mem, int maxord)
   if (maxord > maxord_alloc) {
     IDAProcessError(IDA_mem, IDA_ILL_INPUT, "IDA", "IDASetMaxOrd", MSG_BAD_MAXORD);
     return(IDA_ILL_INPUT);
-  }  
+  }
 
-  IDA_mem->ida_maxord = MIN(maxord,MAXORD_DEFAULT);
+  IDA_mem->ida_maxord = SUNMIN(maxord,MAXORD_DEFAULT);
 
   return(IDA_SUCCESS);
 }
@@ -182,7 +332,7 @@ int IDASetMaxStep(void *ida_mem, realtype hmax)
 
   IDA_mem = (IDAMem) ida_mem;
 
-  if (hmax < 0) {
+  if (hmax < ZERO) {
     IDAProcessError(IDA_mem, IDA_ILL_INPUT, "IDA", "IDASetMaxStep", MSG_NEG_HMAX);
     return(IDA_ILL_INPUT);
   }
@@ -200,6 +350,36 @@ int IDASetMaxStep(void *ida_mem, realtype hmax)
 
 /*-----------------------------------------------------------------*/
 
+int IDASetMinStep(void *ida_mem, realtype hmin)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem == NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDASetMinStep", MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+
+  IDA_mem = (IDAMem) ida_mem;
+
+  if (hmin < ZERO) {
+    IDAProcessError(IDA_mem, IDA_ILL_INPUT, "IDA", "IDASetMinStep",
+                    MSG_NEG_HMIN);
+    return(IDA_ILL_INPUT);
+  }
+
+  /* Passing 0 sets hmin = zero */
+  if (hmin == ZERO) {
+    IDA_mem->ida_hmin = HMIN_DEFAULT;
+    return(IDA_SUCCESS);
+  }
+
+  IDA_mem->ida_hmin = hmin;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
 int IDASetStopTime(void *ida_mem, realtype tstop)
 {
   IDAMem IDA_mem;
@@ -218,14 +398,14 @@ int IDASetStopTime(void *ida_mem, realtype tstop)
   if (IDA_mem->ida_nst > 0) {
 
     if ( (tstop - IDA_mem->ida_tn) * IDA_mem->ida_hh < ZERO ) {
-      IDAProcessError(IDA_mem, IDA_ILL_INPUT, "IDA", "IDASetStopTime", MSG_BAD_TSTOP, IDA_mem->ida_tn);
+      IDAProcessError(IDA_mem, IDA_ILL_INPUT, "IDA", "IDASetStopTime", MSG_BAD_TSTOP, tstop, IDA_mem->ida_tn);
       return(IDA_ILL_INPUT);
     }
 
   }
 
   IDA_mem->ida_tstop = tstop;
-  IDA_mem->ida_tstopset = TRUE;
+  IDA_mem->ida_tstopset = SUNTRUE;
 
   return(IDA_SUCCESS);
 }
@@ -296,15 +476,21 @@ int IDASetMaxNonlinIters(void *ida_mem, int maxcor)
   IDAMem IDA_mem;
 
   if (ida_mem==NULL) {
-    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDASetMaxNonlinIters", MSG_NO_MEM);
-    return (IDA_MEM_NULL);
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA",
+                    "IDASetMaxNonlinIters", MSG_NO_MEM);
+    return(IDA_MEM_NULL);
   }
 
   IDA_mem = (IDAMem) ida_mem;
 
-  IDA_mem->ida_maxcor = maxcor;
+  /* check that the NLS is non-NULL */
+  if (IDA_mem->NLS == NULL) {
+    IDAProcessError(NULL, IDA_MEM_FAIL, "IDA",
+                    "IDASetMaxNonlinIters", MSG_MEM_FAIL);
+    return(IDA_MEM_FAIL);
+  }
 
-  return(IDA_SUCCESS);
+  return(SUNNonlinSolSetMaxIters(IDA_mem->NLS, maxcor));
 }
 
 /*-----------------------------------------------------------------*/
@@ -341,18 +527,18 @@ int IDASetId(void *ida_mem, N_Vector id)
   if (id == NULL) {
     if (IDA_mem->ida_idMallocDone) {
       N_VDestroy(IDA_mem->ida_id);
-      lrw -= lrw1;
-      liw -= liw1;
+      IDA_mem->ida_lrw -= IDA_mem->ida_lrw1;
+      IDA_mem->ida_liw -= IDA_mem->ida_liw1;
     }
-    IDA_mem->ida_idMallocDone = FALSE;    
+    IDA_mem->ida_idMallocDone = SUNFALSE;
     return(IDA_SUCCESS);
   }
 
   if ( !(IDA_mem->ida_idMallocDone) ) {
     IDA_mem->ida_id = N_VClone(id);
-    lrw += lrw1;
-    liw += liw1;
-    IDA_mem->ida_idMallocDone = TRUE;
+    IDA_mem->ida_lrw += IDA_mem->ida_lrw1;
+    IDA_mem->ida_liw += IDA_mem->ida_liw1;
+    IDA_mem->ida_idMallocDone = SUNTRUE;
   }
 
   /* Load the id vector */
@@ -379,11 +565,11 @@ int IDASetConstraints(void *ida_mem, N_Vector constraints)
   if (constraints == NULL) {
     if (IDA_mem->ida_constraintsMallocDone) {
       N_VDestroy(IDA_mem->ida_constraints);
-      lrw -= lrw1;
-      liw -= liw1;
+      IDA_mem->ida_lrw -= IDA_mem->ida_lrw1;
+      IDA_mem->ida_liw -= IDA_mem->ida_liw1;
     }
-    IDA_mem->ida_constraintsMallocDone = FALSE;
-    IDA_mem->ida_constraintsSet = FALSE;
+    IDA_mem->ida_constraintsMallocDone = SUNFALSE;
+    IDA_mem->ida_constraintsSet = SUNFALSE;
     return(IDA_SUCCESS);
   }
 
@@ -401,28 +587,28 @@ int IDASetConstraints(void *ida_mem, N_Vector constraints)
   /*  Check the constraints vector */
 
   temptest = N_VMaxNorm(constraints);
-  if((temptest > TWOPT5) || (temptest < HALF)){ 
+  if((temptest > TWOPT5) || (temptest < HALF)){
     IDAProcessError(IDA_mem, IDA_ILL_INPUT, "IDA", "IDASetConstraints", MSG_BAD_CONSTR);
-    return(IDA_ILL_INPUT); 
+    return(IDA_ILL_INPUT);
   }
 
   if ( !(IDA_mem->ida_constraintsMallocDone) ) {
     IDA_mem->ida_constraints = N_VClone(constraints);
-    lrw += lrw1;
-    liw += liw1;
-    IDA_mem->ida_constraintsMallocDone = TRUE;
+    IDA_mem->ida_lrw += IDA_mem->ida_lrw1;
+    IDA_mem->ida_liw += IDA_mem->ida_liw1;
+    IDA_mem->ida_constraintsMallocDone = SUNTRUE;
   }
 
   /* Load the constraints vector */
 
   N_VScale(ONE, constraints, IDA_mem->ida_constraints);
 
-  IDA_mem->ida_constraintsSet = TRUE;
+  IDA_mem->ida_constraintsSet = SUNTRUE;
 
   return(IDA_SUCCESS);
 }
 
-/* 
+/*
  * IDASetRootDirection
  *
  * Specifies the direction of zero-crossings to be monitored.
@@ -444,7 +630,7 @@ int IDASetRootDirection(void *ida_mem, int *rootdir)
   nrt = IDA_mem->ida_nrtfn;
   if (nrt==0) {
     IDAProcessError(NULL, IDA_ILL_INPUT, "IDA", "IDASetRootDirection", MSG_NO_ROOT);
-    return(IDA_ILL_INPUT);    
+    return(IDA_ILL_INPUT);
   }
 
   for(i=0; i<nrt; i++) IDA_mem->ida_rootdir[i] = rootdir[i];
@@ -471,12 +657,12 @@ int IDASetNoInactiveRootWarn(void *ida_mem)
   IDA_mem = (IDAMem) ida_mem;
 
   IDA_mem->ida_mxgnull = 0;
-  
+
   return(IDA_SUCCESS);
 }
 
 
-/* 
+/*
  * =================================================================
  * IDA IC optional input functions
  * =================================================================
@@ -542,7 +728,7 @@ int IDASetMaxNumJacsIC(void *ida_mem, int maxnj)
    if (maxnj <= 0) {
     IDAProcessError(IDA_mem, IDA_ILL_INPUT, "IDA", "IDASetMaxNumJacsIC", MSG_BAD_MAXNJ);
     return(IDA_ILL_INPUT);
-  } 
+  }
 
   IDA_mem->ida_maxnj = maxnj;
 
@@ -574,6 +760,29 @@ int IDASetMaxNumItersIC(void *ida_mem, int maxnit)
 
 /*-----------------------------------------------------------------*/
 
+int IDASetMaxBacksIC(void *ida_mem, int maxbacks)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem==NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDASetMaxBacksIC", MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+
+  IDA_mem = (IDAMem) ida_mem;
+
+  if (maxbacks <= 0) {
+    IDAProcessError(IDA_mem, IDA_ILL_INPUT, "IDA", "IDASetMaxBacksIC", MSG_IC_BAD_MAXBACKS);
+    return(IDA_ILL_INPUT);
+  }
+
+  IDA_mem->ida_maxbacks = maxbacks;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
 int IDASetLineSearchOffIC(void *ida_mem, booleantype lsoff)
 {
   IDAMem IDA_mem;
@@ -613,36 +822,7 @@ int IDASetStepToleranceIC(void *ida_mem, realtype steptol)
   return(IDA_SUCCESS);
 }
 
-/* 
- * =================================================================
- * Readability constants
- * =================================================================
- */
-
-#define ewt         (IDA_mem->ida_ewt)
-#define kk          (IDA_mem->ida_kk)
-#define hh          (IDA_mem->ida_hh)
-#define h0u         (IDA_mem->ida_h0u)
-#define tn          (IDA_mem->ida_tn)
-#define nbacktr     (IDA_mem->ida_nbacktr)
-#define nst         (IDA_mem->ida_nst)
-#define nre         (IDA_mem->ida_nre)
-#define ncfn        (IDA_mem->ida_ncfn)
-#define netf        (IDA_mem->ida_netf)
-#define nni         (IDA_mem->ida_nni)
-#define nsetups     (IDA_mem->ida_nsetups)
-#define lrw         (IDA_mem->ida_lrw)
-#define liw         (IDA_mem->ida_liw)
-#define kused       (IDA_mem->ida_kused)          
-#define hused       (IDA_mem->ida_hused)         
-#define tolsf       (IDA_mem->ida_tolsf) 
-#define efun        (IDA_mem->ida_efun)
-#define edata       (IDA_mem->ida_edata)
-#define nge         (IDA_mem->ida_nge)
-#define iroots      (IDA_mem->ida_iroots)
-#define ee          (IDA_mem->ida_ee)
-
-/* 
+/*
  * =================================================================
  * IDA optional input functions
  * =================================================================
@@ -659,7 +839,7 @@ int IDAGetNumSteps(void *ida_mem, long int *nsteps)
 
   IDA_mem = (IDAMem) ida_mem;
 
-  *nsteps = nst;
+  *nsteps = IDA_mem->ida_nst;
 
   return(IDA_SUCCESS);
 }
@@ -677,7 +857,7 @@ int IDAGetNumResEvals(void *ida_mem, long int *nrevals)
 
   IDA_mem = (IDAMem) ida_mem;
 
-  *nrevals = nre;
+  *nrevals = IDA_mem->ida_nre;
 
   return(IDA_SUCCESS);
 }
@@ -695,7 +875,7 @@ int IDAGetNumLinSolvSetups(void *ida_mem, long int *nlinsetups)
 
   IDA_mem = (IDAMem) ida_mem;
 
-  *nlinsetups = nsetups;
+  *nlinsetups = IDA_mem->ida_nsetups;
 
   return(IDA_SUCCESS);
 }
@@ -713,7 +893,7 @@ int IDAGetNumErrTestFails(void *ida_mem, long int *netfails)
 
   IDA_mem = (IDAMem) ida_mem;
 
-  *netfails = netf;
+  *netfails = IDA_mem->ida_netf;
 
   return(IDA_SUCCESS);
 }
@@ -731,7 +911,7 @@ int IDAGetNumBacktrackOps(void *ida_mem, long int *nbacktracks)
 
   IDA_mem = (IDAMem) ida_mem;
 
-  *nbacktracks = nbacktr;
+  *nbacktracks = IDA_mem->ida_nbacktr;
 
   return(IDA_SUCCESS);
 }
@@ -741,13 +921,13 @@ int IDAGetNumBacktrackOps(void *ida_mem, long int *nbacktracks)
 int IDAGetConsistentIC(void *ida_mem, N_Vector yy0, N_Vector yp0)
 {
   IDAMem IDA_mem;
-  
+
   if (ida_mem == NULL) {
     IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDAGetConsistentIC", MSG_NO_MEM);
     return (IDA_MEM_NULL);
   }
 
-  IDA_mem = (IDAMem) ida_mem; 
+  IDA_mem = (IDAMem) ida_mem;
 
   if (IDA_mem->ida_kused != 0) {
     IDAProcessError(IDA_mem, IDA_ILL_INPUT, "IDA", "IDAGetConsistentIC", MSG_TOO_LATE);
@@ -773,7 +953,7 @@ int IDAGetLastOrder(void *ida_mem, int *klast)
 
   IDA_mem = (IDAMem) ida_mem;
 
-  *klast = kused;
+  *klast = IDA_mem->ida_kused;
 
   return(IDA_SUCCESS);
 }
@@ -791,7 +971,61 @@ int IDAGetCurrentOrder(void *ida_mem, int *kcur)
 
   IDA_mem = (IDAMem) ida_mem;
 
-  *kcur = kk;
+  *kcur = IDA_mem->ida_kk;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
+int IDAGetCurrentCj(void *ida_mem, realtype *cj)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem==NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDAGetCurrentCjRatio", MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+
+  IDA_mem = (IDAMem) ida_mem;
+
+  *cj = IDA_mem->ida_cj;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
+int IDAGetCurrentY(void *ida_mem, N_Vector *ycur)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem==NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDAGetCurrentY", MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+
+  IDA_mem = (IDAMem) ida_mem;
+
+  *ycur = IDA_mem->ida_yy;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
+int IDAGetCurrentYp(void *ida_mem, N_Vector *ypcur)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem==NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDAGetCurrentYp", MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+
+  IDA_mem = (IDAMem) ida_mem;
+
+  *ypcur = IDA_mem->ida_yp;
 
   return(IDA_SUCCESS);
 }
@@ -809,7 +1043,7 @@ int IDAGetActualInitStep(void *ida_mem, realtype *hinused)
 
   IDA_mem = (IDAMem) ida_mem;
 
-  *hinused = h0u;
+  *hinused = IDA_mem->ida_h0u;
 
   return(IDA_SUCCESS);
 }
@@ -827,7 +1061,7 @@ int IDAGetLastStep(void *ida_mem, realtype *hlast)
 
   IDA_mem = (IDAMem) ida_mem;
 
-  *hlast = hused;
+  *hlast = IDA_mem->ida_hused;
 
   return(IDA_SUCCESS);
 }
@@ -845,7 +1079,7 @@ int IDAGetCurrentStep(void *ida_mem, realtype *hcur)
 
   IDA_mem = (IDAMem) ida_mem;
 
-  *hcur = hh;
+  *hcur = IDA_mem->ida_hh;
 
   return(IDA_SUCCESS);
 }
@@ -863,7 +1097,7 @@ int IDAGetCurrentTime(void *ida_mem, realtype *tcur)
 
   IDA_mem = (IDAMem) ida_mem;
 
-  *tcur = tn;
+  *tcur = IDA_mem->ida_tn;
 
   return(IDA_SUCCESS);
 }
@@ -881,7 +1115,7 @@ int IDAGetTolScaleFactor(void *ida_mem, realtype *tolsfact)
 
   IDA_mem = (IDAMem) ida_mem;
 
-  *tolsfact = tolsf;
+  *tolsfact = IDA_mem->ida_tolsf;
 
   return(IDA_SUCCESS);
 }
@@ -891,15 +1125,15 @@ int IDAGetTolScaleFactor(void *ida_mem, realtype *tolsfact)
 int IDAGetErrWeights(void *ida_mem, N_Vector eweight)
 {
   IDAMem IDA_mem;
-  
+
   if (ida_mem == NULL) {
     IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDAGetErrWeights", MSG_NO_MEM);
     return (IDA_MEM_NULL);
   }
 
-  IDA_mem = (IDAMem) ida_mem; 
+  IDA_mem = (IDAMem) ida_mem;
 
-  N_VScale(ONE, ewt, eweight);
+  N_VScale(ONE, IDA_mem->ida_ewt, eweight);
 
   return(IDA_SUCCESS);
 }
@@ -916,7 +1150,7 @@ int IDAGetEstLocalErrors(void *ida_mem, N_Vector ele)
   }
   IDA_mem = (IDAMem) ida_mem;
 
-  N_VScale(ONE, ee, ele);
+  N_VScale(ONE, IDA_mem->ida_ee, ele);
 
   return(IDA_SUCCESS);
 }
@@ -934,17 +1168,17 @@ int IDAGetWorkSpace(void *ida_mem, long int *lenrw, long int *leniw)
 
   IDA_mem = (IDAMem) ida_mem;
 
-  *leniw = liw;
-  *lenrw = lrw;
+  *leniw = IDA_mem->ida_liw;
+  *lenrw = IDA_mem->ida_lrw;
 
   return(IDA_SUCCESS);
 }
 
 /*-----------------------------------------------------------------*/
 
-int IDAGetIntegratorStats(void *ida_mem, long int *nsteps, long int *nrevals, 
+int IDAGetIntegratorStats(void *ida_mem, long int *nsteps, long int *nrevals,
                           long int *nlinsetups, long int *netfails,
-                          int *klast, int *kcur, realtype *hinused, realtype *hlast, 
+                          int *klast, int *kcur, realtype *hinused, realtype *hlast,
                           realtype *hcur, realtype *tcur)
 {
   IDAMem IDA_mem;
@@ -956,16 +1190,16 @@ int IDAGetIntegratorStats(void *ida_mem, long int *nsteps, long int *nrevals,
 
   IDA_mem = (IDAMem) ida_mem;
 
-  *nsteps     = nst;
-  *nrevals    = nre;
-  *nlinsetups = nsetups;
-  *netfails   = netf;
-  *klast      = kused;
-  *kcur       = kk;
-  *hinused    = h0u;
-  *hlast      = hused;
-  *hcur       = hh;  
-  *tcur       = tn;
+  *nsteps     = IDA_mem->ida_nst;
+  *nrevals    = IDA_mem->ida_nre;
+  *nlinsetups = IDA_mem->ida_nsetups;
+  *netfails   = IDA_mem->ida_netf;
+  *klast      = IDA_mem->ida_kused;
+  *kcur       = IDA_mem->ida_kk;
+  *hinused    = IDA_mem->ida_h0u;
+  *hlast      = IDA_mem->ida_hused;
+  *hcur       = IDA_mem->ida_hh;
+  *tcur       = IDA_mem->ida_tn;
 
   return(IDA_SUCCESS);
 }
@@ -983,7 +1217,7 @@ int IDAGetNumGEvals(void *ida_mem, long int *ngevals)
 
   IDA_mem = (IDAMem) ida_mem;
 
-  *ngevals = nge;
+  *ngevals = IDA_mem->ida_nge;
 
   return(IDA_SUCCESS);
 }
@@ -1004,7 +1238,8 @@ int IDAGetRootInfo(void *ida_mem, int *rootsfound)
 
   nrt = IDA_mem->ida_nrtfn;
 
-  for (i=0; i<nrt; i++) rootsfound[i] = iroots[i];
+  for (i=0; i<nrt; i++)
+    rootsfound[i] = IDA_mem->ida_iroots[i];
 
   return(IDA_SUCCESS);
 }
@@ -1016,20 +1251,21 @@ int IDAGetNumNonlinSolvIters(void *ida_mem, long int *nniters)
   IDAMem IDA_mem;
 
   if (ida_mem==NULL) {
-    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDAGetNumNonlinSolvIters", MSG_NO_MEM);
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA",
+                    "IDAGetNumNonlinSolvIters", MSG_NO_MEM);
     return(IDA_MEM_NULL);
   }
 
   IDA_mem = (IDAMem) ida_mem;
 
-  *nniters = nni;
+  *nniters = IDA_mem->ida_nni;
 
   return(IDA_SUCCESS);
 }
 
 /*-----------------------------------------------------------------*/
 
-int IDAGetNumNonlinSolvConvFails(void *ida_mem, long int *nncfails)
+int IDAGetNumNonlinSolvConvFails(void *ida_mem, long int *nnfails)
 {
   IDAMem IDA_mem;
 
@@ -1040,26 +1276,193 @@ int IDAGetNumNonlinSolvConvFails(void *ida_mem, long int *nncfails)
 
   IDA_mem = (IDAMem) ida_mem;
 
-  *nncfails = ncfn;
+  *nnfails = IDA_mem->ida_nnf;
 
   return(IDA_SUCCESS);
 }
 
 /*-----------------------------------------------------------------*/
 
-int IDAGetNonlinSolvStats(void *ida_mem, long int *nniters, long int *nncfails)
+int IDAGetNonlinSolvStats(void *ida_mem, long int *nniters, long int *nnfails)
 {
   IDAMem IDA_mem;
 
   if (ida_mem==NULL) {
-    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDAGetNonlinSolvStats", MSG_NO_MEM);
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA",
+                    "IDAGetNonlinSolvStats", MSG_NO_MEM);
     return(IDA_MEM_NULL);
   }
 
   IDA_mem = (IDAMem) ida_mem;
 
-  *nniters  = nni;
-  *nncfails = ncfn;
+  *nniters = IDA_mem->ida_nni;
+  *nnfails = IDA_mem->ida_nnf;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
+int IDAGetNumStepSolveFails(void *ida_mem, long int *nncfails)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem==NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDAGetNumStepSolveFails",
+                    MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+
+  IDA_mem = (IDAMem) ida_mem;
+
+  *nncfails = IDA_mem->ida_ncfn;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
+int IDAPrintAllStats(void *ida_mem, FILE *outfile, SUNOutputFormat fmt)
+{
+  IDAMem IDA_mem;
+  IDALsMem idals_mem;
+
+  if (ida_mem == NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDAPrintAllStats",
+                    MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+
+  IDA_mem = (IDAMem) ida_mem;
+
+  switch(fmt)
+  {
+  case SUN_OUTPUTFORMAT_TABLE:
+    /* step and method stats */
+    fprintf(outfile, "Current time                 = %"RSYM"\n", IDA_mem->ida_tn);
+    fprintf(outfile, "Steps                        = %ld\n", IDA_mem->ida_nst);
+    fprintf(outfile, "Error test fails             = %ld\n", IDA_mem->ida_netf);
+    fprintf(outfile, "NLS step fails               = %ld\n", IDA_mem->ida_ncfn);
+    fprintf(outfile, "Initial step size            = %"RSYM"\n", IDA_mem->ida_h0u);
+    fprintf(outfile, "Last step size               = %"RSYM"\n", IDA_mem->ida_hused);
+    fprintf(outfile, "Current step size            = %"RSYM"\n", IDA_mem->ida_hh);
+    fprintf(outfile, "Last method order            = %d\n", IDA_mem->ida_kused);
+    fprintf(outfile, "Current method order         = %d\n", IDA_mem->ida_kk);
+
+    /* function evaluations */
+    fprintf(outfile, "Residual fn evals            = %ld\n", IDA_mem->ida_nre);
+
+    /* IC calculation stats */
+    fprintf(outfile, "IC linesearch backtrack ops  = %d\n", IDA_mem->ida_nbacktr);
+
+    /* nonlinear solver stats */
+    fprintf(outfile, "NLS iters                    = %ld\n", IDA_mem->ida_nni);
+    fprintf(outfile, "NLS fails                    = %ld\n", IDA_mem->ida_nnf);
+    if (IDA_mem->ida_nst > 0)
+    {
+      fprintf(outfile, "NLS iters per step           = %"RSYM"\n",
+              (realtype) IDA_mem->ida_nre / (realtype) IDA_mem->ida_nst);
+    }
+
+    /* linear solver stats */
+    fprintf(outfile, "LS setups                    = %ld\n", IDA_mem->ida_nsetups);
+    if (IDA_mem->ida_lmem)
+    {
+      idals_mem = (IDALsMem) (IDA_mem->ida_lmem);
+      fprintf(outfile, "Jac fn evals                 = %ld\n", idals_mem->nje);
+      fprintf(outfile, "LS residual fn evals         = %ld\n", idals_mem->nreDQ);
+      fprintf(outfile, "Prec setup evals             = %ld\n", idals_mem->npe);
+      fprintf(outfile, "Prec solves                  = %ld\n", idals_mem->nps);
+      fprintf(outfile, "LS iters                     = %ld\n", idals_mem->nli);
+      fprintf(outfile, "LS fails                     = %ld\n", idals_mem->ncfl);
+      fprintf(outfile, "Jac-times setups             = %ld\n", idals_mem->njtsetup);
+      fprintf(outfile, "Jac-times evals              = %ld\n", idals_mem->njtimes);
+      if (IDA_mem->ida_nni > 0)
+      {
+        fprintf(outfile, "LS iters per NLS iter        = %"RSYM"\n",
+                (realtype) idals_mem->nli / (realtype) IDA_mem->ida_nni);
+        fprintf(outfile, "Jac evals per NLS iter       = %"RSYM"\n",
+                (realtype) idals_mem->nje / (realtype) IDA_mem->ida_nni);
+        fprintf(outfile, "Prec evals per NLS iter      = %"RSYM"\n",
+                (realtype) idals_mem->npe / (realtype) IDA_mem->ida_nni);
+      }
+    }
+
+    /* rootfinding stats */
+    fprintf(outfile, "Root fn evals                = %ld\n", IDA_mem->ida_nge);
+    break;
+
+  case SUN_OUTPUTFORMAT_CSV:
+    /* step and method stats */
+    fprintf(outfile, "Time,%"RSYM, IDA_mem->ida_tn);
+    fprintf(outfile, ",Steps,%ld", IDA_mem->ida_nst);
+    fprintf(outfile, ",Error test fails,%ld", IDA_mem->ida_netf);
+    fprintf(outfile, ",NLS step fails,%ld", IDA_mem->ida_ncfn);
+    fprintf(outfile, ",Initial step size,%"RSYM, IDA_mem->ida_h0u);
+    fprintf(outfile, ",Last step size,%"RSYM, IDA_mem->ida_hused);
+    fprintf(outfile, ",Current step size,%"RSYM, IDA_mem->ida_hh);
+    fprintf(outfile, ",Last method order,%d", IDA_mem->ida_kused);
+    fprintf(outfile, ",Current method order,%d", IDA_mem->ida_kk);
+
+    /* function evaluations */
+    fprintf(outfile, ",Residual fn evals,%ld", IDA_mem->ida_nre);
+
+    /* IC calculation stats */
+    fprintf(outfile, ",IC linesearch backtrack ops,%d", IDA_mem->ida_nbacktr);
+
+    /* nonlinear solver stats */
+    fprintf(outfile, ",NLS iters,%ld", IDA_mem->ida_nni);
+    fprintf(outfile, ",NLS fails,%ld", IDA_mem->ida_nnf);
+    if (IDA_mem->ida_nst > 0)
+    {
+      fprintf(outfile, ",NLS iters per step,%"RSYM,
+              (realtype) IDA_mem->ida_nre / (realtype) IDA_mem->ida_nst);
+    }
+    else
+    {
+      fprintf(outfile, ",NLS iters per step,0");
+    }
+
+    /* linear solver stats */
+    fprintf(outfile, ",LS setups,%ld", IDA_mem->ida_nsetups);
+    if (IDA_mem->ida_lmem)
+    {
+      idals_mem = (IDALsMem) (IDA_mem->ida_lmem);
+      fprintf(outfile, ",Jac fn evals,%ld", idals_mem->nje);
+      fprintf(outfile, ",LS residual evals,%ld", idals_mem->nreDQ);
+      fprintf(outfile, ",Prec setup evals,%ld", idals_mem->npe);
+      fprintf(outfile, ",Prec solves,%ld", idals_mem->nps);
+      fprintf(outfile, ",LS iters,%ld", idals_mem->nli);
+      fprintf(outfile, ",LS fails,%ld", idals_mem->ncfl);
+      fprintf(outfile, ",Jac-times setups,%ld", idals_mem->njtsetup);
+      fprintf(outfile, ",Jac-times evals,%ld", idals_mem->njtimes);
+      if (IDA_mem->ida_nni > 0)
+      {
+        fprintf(outfile, ",LS iters per NLS iter,%"RSYM,
+                (realtype) idals_mem->nli / (realtype) IDA_mem->ida_nni);
+        fprintf(outfile, ",Jac evals per NLS iter,%"RSYM,
+                (realtype) idals_mem->nje / (realtype) IDA_mem->ida_nni);
+        fprintf(outfile, ",Prec evals per NLS iter,%"RSYM,
+                (realtype) idals_mem->npe / (realtype) IDA_mem->ida_nni);
+      }
+      else
+      {
+        fprintf(outfile, ",LS iters per NLS iter,0");
+        fprintf(outfile, ",Jac evals per NLS iter,0");
+        fprintf(outfile, ",Prec evals per NLS iter,0");
+      }
+    }
+
+    /* rootfinding stats */
+    fprintf(outfile, ",Root fn evals,%ld", IDA_mem->ida_nge);
+    fprintf(outfile, "\n");
+    break;
+
+  default:
+    IDAProcessError(IDA_mem, IDA_ILL_INPUT, "IDA", "IDAPrintAllStats",
+                    "Invalid formatting option.");
+    return(IDA_ILL_INPUT);
+  }
 
   return(IDA_SUCCESS);
 }
@@ -1142,11 +1545,15 @@ char *IDAGetReturnFlagName(long int flag)
   case IDA_LINESEARCH_FAIL:
     sprintf(name,"IDA_LINESEARCH_FAIL");
     break;
-
+  case IDA_NLS_SETUP_FAIL:
+    sprintf(name,"IDA_NLS_SETUP_FAIL");
+    break;
+  case IDA_NLS_FAIL:
+    sprintf(name,"IDA_NLS_FAIL");
+    break;
   default:
     sprintf(name,"NONE");
   }
 
   return(name);
 }
-
