@@ -1,15 +1,16 @@
-/*
- * -----------------------------------------------------------------
- * $Revision: 1.8 $
- * $Date: 2010/12/01 22:43:33 $
- * -----------------------------------------------------------------
+/* -----------------------------------------------------------------
  * Programmer(s): Allan Taylor, Alan Hindmarsh, Radu Serban, and
- *                Aaron Collier @ LLNL
+ *                Aaron Collier, Shelby Lockhart @ LLNL
  * -----------------------------------------------------------------
- * Copyright (c) 2002, The Regents of the University of California.
- * Produced at the Lawrence Livermore National Laboratory.
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
+ * and Southern Methodist University.
  * All rights reserved.
- * For details, see the LICENSE file.
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
  * -----------------------------------------------------------------
  * This is the implementation file for the optional input and output
  * functions for the KINSOL solver.
@@ -20,6 +21,7 @@
 #include <stdlib.h>
 
 #include "kinsol_impl.h"
+#include "kinsol_ls_impl.h"
 #include <sundials/sundials_types.h>
 #include <sundials/sundials_math.h>
 
@@ -33,18 +35,13 @@
 #define TWO       RCONST(2.0)
 #define TWOPT5    RCONST(2.5)
 
-#define liw  (kin_mem->kin_liw)
-#define lrw  (kin_mem->kin_lrw)
-#define liw1 (kin_mem->kin_liw1)
-#define lrw1 (kin_mem->kin_lrw1)
-
-/* 
+/*
  * =================================================================
  * KINSOL optional input functions
  * =================================================================
  */
 
-/* 
+/*
  * -----------------------------------------------------------------
  * KINSetErrHandlerFn
  * -----------------------------------------------------------------
@@ -88,8 +85,6 @@ int KINSetErrFile(void *kinmem, FILE *errfp)
   return(KIN_SUCCESS);
 }
 
-#define errfp (kin_mem->kin_errfp)
-
 /*
  * -----------------------------------------------------------------
  * Function : KINSetPrintLevel
@@ -117,7 +112,7 @@ int KINSetPrintLevel(void *kinmem, int printfl)
   return(KIN_SUCCESS);
 }
 
-/* 
+/*
  * -----------------------------------------------------------------
  * KINSetInfoHandlerFn
  * -----------------------------------------------------------------
@@ -144,6 +139,7 @@ int KINSetInfoHandlerFn(void *kinmem, KINInfoHandlerFn ihfun, void *ih_data)
 /*
  * -----------------------------------------------------------------
  * Function : KINSetInfoFile
+ * DEPRECATED: use SUNLogger_SetInfoFilename instead
  * -----------------------------------------------------------------
  */
 
@@ -162,6 +158,31 @@ int KINSetInfoFile(void *kinmem, FILE *infofp)
   return(KIN_SUCCESS);
 }
 
+
+/*
+ * -----------------------------------------------------------------
+ * Function : KINSetDebugFile
+ * DEPRECATED: use SUNLogger_SetDebugFilename instead
+ * -----------------------------------------------------------------
+ */
+
+int KINSetDebugFile(void *kinmem, FILE *debugfp)
+{
+  KINMem kin_mem;
+
+  if (kinmem == NULL) {
+    KINProcessError(NULL, KIN_MEM_NULL, "KINSOL", "KINSetDebugFile",
+                    MSG_NO_MEM);
+    return(KIN_MEM_NULL);
+  }
+
+  kin_mem = (KINMem) kinmem;
+  kin_mem->kin_debugfp = debugfp;
+
+  return(KIN_SUCCESS);
+}
+
+
 /*
  * -----------------------------------------------------------------
  * Function : KINSetUserData
@@ -183,6 +204,190 @@ int KINSetUserData(void *kinmem, void *user_data)
   return(KIN_SUCCESS);
 }
 
+/*
+ * -----------------------------------------------------------------
+ * Function : KINSetDamping
+ * -----------------------------------------------------------------
+ */
+
+int KINSetDamping(void *kinmem, realtype beta)
+{
+  KINMem kin_mem;
+
+  if (kinmem == NULL) {
+    KINProcessError(NULL, KIN_MEM_NULL, "KINSOL", "KINSetDamping", MSG_NO_MEM);
+    return(KIN_MEM_NULL);
+  }
+
+  kin_mem = (KINMem) kinmem;
+
+  /* check for illegal input value */
+  if (beta <= ZERO) {
+    KINProcessError(NULL, KIN_ILL_INPUT, "KINSOL", "KINSetDamping",
+                    "beta <= 0 illegal");
+    return(KIN_ILL_INPUT);
+  }
+
+  if (beta < ONE) {
+    /* enable damping */
+    kin_mem->kin_beta    = beta;
+    kin_mem->kin_damping = SUNTRUE;
+  } else {
+    /* disable damping */
+    kin_mem->kin_beta    = ONE;
+    kin_mem->kin_damping = SUNFALSE;
+  }
+
+  return(KIN_SUCCESS);
+}
+
+
+/*
+ * -----------------------------------------------------------------
+ * Function : KINSetMAA
+ * -----------------------------------------------------------------
+ */
+
+int KINSetMAA(void *kinmem, long int maa)
+{
+  KINMem kin_mem;
+
+  if (kinmem == NULL) {
+    KINProcessError(NULL, KIN_MEM_NULL, "KINSOL", "KINSetMAA", MSG_NO_MEM);
+    return(KIN_MEM_NULL);
+  }
+
+  kin_mem = (KINMem) kinmem;
+
+  if (maa < 0) {
+    KINProcessError(NULL, KIN_ILL_INPUT, "KINSOL", "KINSetMAA", MSG_BAD_MAA);
+    return(KIN_ILL_INPUT);
+  }
+
+  if (maa > kin_mem->kin_mxiter) maa = kin_mem->kin_mxiter;
+
+  kin_mem->kin_m_aa = maa;
+
+  return(KIN_SUCCESS);
+}
+
+
+/*
+ * -----------------------------------------------------------------
+ * Function : KINSetDelayAA
+ * -----------------------------------------------------------------
+ */
+
+int KINSetDelayAA(void *kinmem, long int delay)
+{
+  KINMem kin_mem;
+
+  if (kinmem == NULL) {
+    KINProcessError(NULL, KIN_MEM_NULL, "KINSOL", "KINSetDelayAA", MSG_NO_MEM);
+    return(KIN_MEM_NULL);
+  }
+
+  kin_mem = (KINMem) kinmem;
+
+  /* check for illegal input value */
+  if (delay < 0) {
+    KINProcessError(NULL, KIN_ILL_INPUT, "KINSOL", "KINSetDelayAA",
+                    "delay < 0 illegal");
+    return(KIN_ILL_INPUT);
+  }
+
+  kin_mem->kin_delay_aa = delay;
+
+  return(KIN_SUCCESS);
+}
+
+
+/*
+ * -----------------------------------------------------------------
+ * Function : KINSetOrthAA
+ * -----------------------------------------------------------------
+ */
+
+int KINSetOrthAA(void *kinmem, int orthaa)
+{
+  KINMem kin_mem;
+
+  if (kinmem == NULL) {
+    KINProcessError(NULL, KIN_MEM_NULL, "KINSOL", "KINSetOrthAA", MSG_NO_MEM);
+    return(KIN_MEM_NULL);
+  }
+
+  kin_mem = (KINMem) kinmem;
+
+  if ((orthaa < KIN_ORTH_MGS) || (orthaa > KIN_ORTH_DCGS2)) {
+    KINProcessError(kin_mem, KIN_ILL_INPUT, "KINSOL", "KINSetOrthAA", MSG_BAD_ORTHAA);
+    return(KIN_ILL_INPUT);
+  }
+
+  kin_mem->kin_orth_aa = orthaa;
+
+  return(KIN_SUCCESS);
+}
+
+/*
+ * -----------------------------------------------------------------
+ * Function : KINSetDampingAA
+ * -----------------------------------------------------------------
+ */
+
+int KINSetDampingAA(void *kinmem, realtype beta)
+{
+  KINMem kin_mem;
+
+  if (kinmem == NULL) {
+    KINProcessError(NULL, KIN_MEM_NULL, "KINSOL", "KINSetDampingAA", MSG_NO_MEM);
+    return(KIN_MEM_NULL);
+  }
+
+  kin_mem = (KINMem) kinmem;
+
+  /* check for illegal input value */
+  if (beta <= ZERO) {
+    KINProcessError(NULL, KIN_ILL_INPUT, "KINSOL", "KINSetDampingAA",
+                    "beta <= 0 illegal");
+    return(KIN_ILL_INPUT);
+  }
+
+  if (beta < ONE) {
+    /* enable damping */
+    kin_mem->kin_beta_aa    = beta;
+    kin_mem->kin_damping_aa = SUNTRUE;
+  } else {
+    /* disable damping */
+    kin_mem->kin_beta_aa    = ONE;
+    kin_mem->kin_damping_aa = SUNFALSE;
+  }
+
+  return(KIN_SUCCESS);
+}
+
+/*
+ * -----------------------------------------------------------------
+ * Function : KINSetReturnNewest
+ * -----------------------------------------------------------------
+ */
+
+int KINSetReturnNewest(void *kinmem, booleantype ret_newest)
+{
+  KINMem kin_mem;
+
+  if (kinmem == NULL) {
+    KINProcessError(NULL, KIN_MEM_NULL, "KINSOL", "KINSetReturnNewest", MSG_NO_MEM);
+    return(KIN_MEM_NULL);
+  }
+
+  kin_mem = (KINMem) kinmem;
+
+  kin_mem->kin_ret_newest = ret_newest;
+
+  return(KIN_SUCCESS);
+}
+
 /*
  * -----------------------------------------------------------------
  * Function : KINSetNumMaxIters
@@ -276,7 +481,7 @@ int KINSetMaxSetupCalls(void *kinmem, long int msbset)
     KINProcessError(NULL, KIN_ILL_INPUT, "KINSOL", "KINSetMaxSetupCalls", MSG_BAD_MSBSET);
     return(KIN_ILL_INPUT);
   }
-  
+
   if (msbset == 0)
     kin_mem->kin_msbset = MSBSET_DEFAULT;
   else
@@ -306,7 +511,7 @@ int KINSetMaxSubSetupCalls(void *kinmem, long int msbsetsub)
     KINProcessError(NULL, KIN_ILL_INPUT, "KINSOL", "KINSetMaxSubSetupCalls", MSG_BAD_MSBSETSUB);
     return(KIN_ILL_INPUT);
   }
-  
+
   if (msbsetsub == 0)
     kin_mem->kin_msbset_sub = MSBSET_SUB_DEFAULT;
   else
@@ -332,8 +537,8 @@ int KINSetEtaForm(void *kinmem, int etachoice)
 
   kin_mem = (KINMem) kinmem;
 
-  if ((etachoice != KIN_ETACONSTANT) && 
-      (etachoice != KIN_ETACHOICE1)  && 
+  if ((etachoice != KIN_ETACONSTANT) &&
+      (etachoice != KIN_ETACHOICE1)  &&
       (etachoice != KIN_ETACHOICE2)) {
     KINProcessError(NULL, KIN_ILL_INPUT, "KINSOL", "KINSetEtaForm", MSG_BAD_ETACHOICE);
     return(KIN_ILL_INPUT);
@@ -396,8 +601,8 @@ int KINSetEtaParams(void *kinmem, realtype egamma, realtype ealpha)
       KINProcessError(NULL, KIN_ILL_INPUT, "KINSOL", "KINSetEtaParams", MSG_BAD_ALPHA);
       return(KIN_ILL_INPUT);
     }
-  
-  if (ealpha == ZERO) 
+
+  if (ealpha == ZERO)
     kin_mem->kin_eta_alpha = TWO;
   else
     kin_mem->kin_eta_alpha = ealpha;
@@ -440,7 +645,7 @@ int KINSetResMonParams(void *kinmem, realtype omegamin, realtype omegamax)
     return(KIN_ILL_INPUT);
   }
 
-  if (omegamin == ZERO) 
+  if (omegamin == ZERO)
     kin_mem->kin_omega_min = OMEGA_MIN;
   else
     kin_mem->kin_omega_min = omegamin;
@@ -547,9 +752,9 @@ int KINSetMaxNewtonStep(void *kinmem, realtype mxnewtstep)
   }
 
   /* Note: passing a value of 0.0 will use the default
-     value (computed in KINSolinit) */
+     value (computed in KINSolInit) */
 
-  kin_mem->kin_mxnewtstep = mxnewtstep;
+  kin_mem->kin_mxnstepin = mxnewtstep;
 
   return(KIN_SUCCESS);
 }
@@ -610,9 +815,9 @@ int KINSetRelErrFunc(void *kinmem, realtype relfunc)
 
   if (relfunc == ZERO) {
     uround = kin_mem->kin_uround;
-    kin_mem->kin_sqrt_relfunc = RSqrt(uround);
+    kin_mem->kin_sqrt_relfunc = SUNRsqrt(uround);
   } else {
-    kin_mem->kin_sqrt_relfunc = RSqrt(relfunc);
+    kin_mem->kin_sqrt_relfunc = SUNRsqrt(relfunc);
   }
 
   return(KIN_SUCCESS);
@@ -643,7 +848,7 @@ int KINSetFuncNormTol(void *kinmem, realtype fnormtol)
 
   if (fnormtol == ZERO) {
     uround = kin_mem->kin_uround;
-    kin_mem->kin_fnormtol = RPowerR(uround,ONETHIRD);
+    kin_mem->kin_fnormtol = SUNRpowerR(uround,ONETHIRD);
   } else {
     kin_mem->kin_fnormtol = fnormtol;
   }
@@ -676,7 +881,7 @@ int KINSetScaledStepTol(void *kinmem, realtype scsteptol)
 
   if (scsteptol == ZERO) {
     uround = kin_mem->kin_uround;
-    kin_mem->kin_scsteptol = RPowerR(uround,TWOTHIRDS);
+    kin_mem->kin_scsteptol = SUNRpowerR(uround,TWOTHIRDS);
   } else {
     kin_mem->kin_scsteptol = scsteptol;
   }
@@ -705,26 +910,26 @@ int KINSetConstraints(void *kinmem, N_Vector constraints)
   if (constraints == NULL) {
     if (kin_mem->kin_constraintsSet) {
       N_VDestroy(kin_mem->kin_constraints);
-      lrw -= lrw1;
-      liw -= liw1;
+      kin_mem->kin_lrw -= kin_mem->kin_lrw1;
+      kin_mem->kin_liw -= kin_mem->kin_liw1;
     }
-    kin_mem->kin_constraintsSet = FALSE;
+    kin_mem->kin_constraintsSet = SUNFALSE;
     return(KIN_SUCCESS);
   }
 
   /* Check the constraints vector */
 
   temptest = N_VMaxNorm(constraints);
-  if (temptest > TWOPT5){ 
+  if (temptest > TWOPT5){
     KINProcessError(NULL, KIN_ILL_INPUT, "KINSOL", "KINSetConstraints", MSG_BAD_CONSTRAINTS);
-    return(KIN_ILL_INPUT); 
+    return(KIN_ILL_INPUT);
   }
 
   if (!kin_mem->kin_constraintsSet) {
     kin_mem->kin_constraints = N_VClone(constraints);
-    lrw += lrw1;
-    liw += liw1;
-    kin_mem->kin_constraintsSet = TRUE;
+    kin_mem->kin_lrw += kin_mem->kin_lrw1;
+    kin_mem->kin_liw += kin_mem->kin_liw1;
+    kin_mem->kin_constraintsSet = SUNTRUE;
   }
 
   /* Load the constraint vector */
@@ -761,24 +966,10 @@ int KINSetSysFunc(void *kinmem, KINSysFn func)
   return(KIN_SUCCESS);
 }
 
-/* 
- * =================================================================
- * Readability constants
- * =================================================================
- */
 
-#define nni (kin_mem->kin_nni)
-#define nfe (kin_mem->kin_nfe)
-#define nbcf (kin_mem->kin_nbcf)  
-#define nbktrk (kin_mem->kin_nbktrk)
-#define stepl (kin_mem->kin_stepl)
-#define fnorm (kin_mem->kin_fnorm)
-#define liw (kin_mem->kin_liw)
-#define lrw (kin_mem->kin_lrw)
-
-/* 
+/*
  * =================================================================
- * KINSOL optional input functions
+ * KINSOL optional output functions
  * =================================================================
  */
 
@@ -799,8 +990,8 @@ int KINGetWorkSpace(void *kinmem, long int *lenrw, long int *leniw)
 
   kin_mem = (KINMem) kinmem;
 
-  *lenrw = lrw;
-  *leniw = liw;
+  *lenrw = kin_mem->kin_lrw;
+  *leniw = kin_mem->kin_liw;
 
   return(KIN_SUCCESS);
 }
@@ -821,7 +1012,7 @@ int KINGetNumNonlinSolvIters(void *kinmem, long int *nniters)
   }
 
   kin_mem = (KINMem) kinmem;
-  *nniters = nni;
+  *nniters = kin_mem->kin_nni;
 
   return(KIN_SUCCESS);
 }
@@ -842,7 +1033,7 @@ int KINGetNumFuncEvals(void *kinmem, long int *nfevals)
   }
 
   kin_mem = (KINMem) kinmem;
-  *nfevals = nfe;
+  *nfevals = kin_mem->kin_nfe;
 
   return(KIN_SUCCESS);
 }
@@ -863,7 +1054,7 @@ int KINGetNumBetaCondFails(void *kinmem, long int *nbcfails)
   }
 
   kin_mem = (KINMem) kinmem;
-  *nbcfails = nbcf;
+  *nbcfails = kin_mem->kin_nbcf;
 
   return(KIN_SUCCESS);
 }
@@ -884,7 +1075,7 @@ int KINGetNumBacktrackOps(void *kinmem, long int *nbacktr)
   }
 
   kin_mem = (KINMem) kinmem;
-  *nbacktr = nbktrk;
+  *nbacktr = kin_mem->kin_nbktrk;
 
   return(KIN_SUCCESS);
 }
@@ -905,7 +1096,7 @@ int KINGetFuncNorm(void *kinmem, realtype *funcnorm)
   }
 
   kin_mem = (KINMem) kinmem;
-  *funcnorm = fnorm;
+  *funcnorm = kin_mem->kin_fnorm;
 
   return(KIN_SUCCESS);
 }
@@ -926,7 +1117,107 @@ int KINGetStepLength(void *kinmem, realtype *steplength)
   }
 
   kin_mem = (KINMem) kinmem;
-  *steplength = stepl;
+  *steplength = kin_mem->kin_stepl;
+
+  return(KIN_SUCCESS);
+}
+
+/*
+ * -----------------------------------------------------------------
+ * Function : KINPrintAllStats
+ * -----------------------------------------------------------------
+ */
+
+int KINPrintAllStats(void *kinmem, FILE* outfile, SUNOutputFormat fmt)
+{
+  KINMem kin_mem;
+  KINLsMem kinls_mem;
+
+  if (kinmem == NULL) {
+    KINProcessError(NULL, KIN_MEM_NULL, "KINSOL", "KINPrintAllStats",
+                    MSG_NO_MEM);
+    return(KIN_MEM_NULL);
+  }
+
+  kin_mem = (KINMem) kinmem;
+
+  switch(fmt)
+  {
+  case SUN_OUTPUTFORMAT_TABLE:
+    /* main solver stats */
+    fprintf(outfile, "Nonlinear iters         = %li\n", kin_mem->kin_nni);
+    fprintf(outfile, "Nonlinear fn evals      = %li\n", kin_mem->kin_nfe);
+    fprintf(outfile, "Beta condition fails    = %li\n", kin_mem->kin_nbcf);
+    fprintf(outfile, "Backtrack operations    = %li\n", kin_mem->kin_nbktrk);
+    fprintf(outfile, "Nonlinear fn norm       = %"RSYM"\n", kin_mem->kin_fnorm);
+    fprintf(outfile, "Step length             = %"RSYM"\n", kin_mem->kin_stepl);
+
+    /* linear solver stats */
+    if (kin_mem->kin_lmem)
+    {
+      kinls_mem = (KINLsMem) (kin_mem->kin_lmem);
+      fprintf(outfile, "Jac fn evals            = %ld\n", kinls_mem->nje);
+      fprintf(outfile, "LS Nonlinear fn evals   = %ld\n", kinls_mem->nfeDQ);
+      fprintf(outfile, "Prec setup evals        = %ld\n", kinls_mem->npe);
+      fprintf(outfile, "Prec solves             = %ld\n", kinls_mem->nps);
+      fprintf(outfile, "LS iters                = %ld\n", kinls_mem->nli);
+      fprintf(outfile, "LS fails                = %ld\n", kinls_mem->ncfl);
+      fprintf(outfile, "Jac-times evals         = %ld\n", kinls_mem->njtimes);
+      if (kin_mem->kin_nni > 0)
+      {
+        fprintf(outfile, "LS iters per NLS iter   = %"RSYM"\n",
+                (realtype) kinls_mem->nli / (realtype) kin_mem->kin_nni);
+        fprintf(outfile, "Jac evals per NLS iter  = %"RSYM"\n",
+                (realtype) kinls_mem->nje / (realtype) kin_mem->kin_nni);
+        fprintf(outfile, "Prec evals per NLS iter = %"RSYM"\n",
+                (realtype) kinls_mem->npe / (realtype) kin_mem->kin_nni);
+      }
+    }
+
+    break;
+  case SUN_OUTPUTFORMAT_CSV:
+    /* main solver stats */
+    fprintf(outfile, "Nonlinear iters,%li", kin_mem->kin_nni);
+    fprintf(outfile, ",Nonlinear fn evals,%li", kin_mem->kin_nfe);
+    fprintf(outfile, ",Beta condition fails,%li", kin_mem->kin_nbcf);
+    fprintf(outfile, ",Backtrack operations,%li", kin_mem->kin_nbktrk);
+    fprintf(outfile, ",Nonlinear fn norm,%"RSYM, kin_mem->kin_fnorm);
+    fprintf(outfile, ",Step length,%"RSYM, kin_mem->kin_stepl);
+
+    /* linear solver stats */
+    if (kin_mem->kin_lmem)
+    {
+      kinls_mem = (KINLsMem) (kin_mem->kin_lmem);
+      fprintf(outfile, ",Jac fn evals,%ld", kinls_mem->nje);
+      fprintf(outfile, ",LS Nonlinear fn evals,%ld", kinls_mem->nfeDQ);
+      fprintf(outfile, ",Prec setup evals,%ld", kinls_mem->npe);
+      fprintf(outfile, ",Prec solves,%ld", kinls_mem->nps);
+      fprintf(outfile, ",LS iters,%ld", kinls_mem->nli);
+      fprintf(outfile, ",LS fails,%ld", kinls_mem->ncfl);
+      fprintf(outfile, ",Jac-times evals,%ld", kinls_mem->njtimes);
+      if (kin_mem->kin_nni > 0)
+      {
+        fprintf(outfile, ",LS iters per NLS iter,%"RSYM,
+                (realtype) kinls_mem->nli / (realtype) kin_mem->kin_nni);
+        fprintf(outfile, ",Jac evals per NLS iter,%"RSYM,
+                (realtype) kinls_mem->nje / (realtype) kin_mem->kin_nni);
+        fprintf(outfile, ",Prec evals per NLS iter,%"RSYM,
+                (realtype) kinls_mem->npe / (realtype) kin_mem->kin_nni);
+      }
+      else
+      {
+        fprintf(outfile, ",LS iters per NLS iter,0");
+        fprintf(outfile, ",Jac evals per NLS iter,0");
+        fprintf(outfile, ",Prec evals per NLS iter,0");
+      }
+    }
+    fprintf(outfile, "\n");
+    break;
+  default:
+    KINProcessError(kin_mem, KIN_ILL_INPUT, "KINSOL", "KINPrintAllStats",
+                    "Invalid formatting option.");
+    return(KIN_ILL_INPUT);
+  }
 
   return(KIN_SUCCESS);
 }
